var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/backend.ts
var backend_exports = {};
__export(backend_exports, {
  BullmqBackend: () => BullmqBackend
});
module.exports = __toCommonJS(backend_exports);
var import_lodash = __toESM(require("lodash"));
var import_bullmq = require("bullmq");
var import_debug2 = __toESM(require("debug"));

// src/worker.ts
var import_reflect_metadata = require("reflect-metadata");
var import_debug = __toESM(require("debug"));
var debug = (0, import_debug.default)("hyperworker:worker");
var K_QUEUE_NAME = "hyperworker:queue-name";
var K_PRODUCER_ONLY = "hyperworker:queue-producer-only";
var _HyperWorker = class {
  static clear() {
    this.logger = null;
    this.backend = null;
    this.inprocess = false;
    this.enqueue = null;
    this.reportError = (_err, _tags) => {
    };
  }
  static initWorker({ logger }) {
    this.logger = logger;
  }
  static setMode({
    inprocess,
    reportError
  }) {
    this.inprocess = inprocess;
    if (reportError) {
      this.reportError = reportError;
    }
  }
  static setTest(reportError) {
    this.setMode({ inprocess: true, reportError });
  }
  static get queueName() {
    return Reflect.getOwnMetadata(K_QUEUE_NAME, this) || this.name.toLowerCase();
  }
  static get producerOnly() {
    return Reflect.getOwnMetadata(K_PRODUCER_ONLY, this) || false;
  }
  static async performNow(args) {
    const w = new this();
    return w.perform({ data: args });
  }
  static async performLater(_args) {
    if (this.inprocess) {
      debug(`${this.name}: perform now '%o'`, _args);
      return this.performNow(_args);
    }
    debug(`${this.name}: perform later '%o'`, _args);
    return this.enqueue(_args, {});
  }
  static async jobCounts() {
    return await this.queue.getJobCounts();
  }
  get logger() {
    return _HyperWorker.logger;
  }
  perform(_args) {
  }
};
var HyperWorker = _HyperWorker;
HyperWorker.inprocess = false;
HyperWorker.logger = null;
HyperWorker.backend = null;
HyperWorker.enqueue = null;
HyperWorker.queue = null;
HyperWorker.reportError = (_err, _tags) => {
};

// src/backend.ts
var debug2 = (0, import_debug2.default)("hyperworker:backend");
var sleep = (t) => new Promise((resolve, _reject) => setTimeout(() => resolve(true), t));
var defaultWorkerOpts = {
  lockDuration: 3e4
};
var defaultQueueOpts = {
  streams: {
    events: { maxLen: 1e3 }
  },
  defaultJobOptions: {
    removeOnComplete: true,
    removeOnFail: { count: 100 },
    stackTraceLimit: 5
  }
};
var BullmqBackend = class {
  constructor(workers, connection, opts) {
    this.connection = connection;
    this.workers = workers;
    this.opts = {
      workerOpts: import_lodash.default.merge({}, defaultWorkerOpts, opts == null ? void 0 : opts.workerOpts),
      queueOpts: import_lodash.default.merge({}, defaultQueueOpts, opts == null ? void 0 : opts.queueOpts)
    };
    debug2("opts: %o", this.opts);
    this.bmqueues = workers.map((Klass) => {
      debug2(`adding queue: ${Klass.queueName} from class ${Klass.name}`);
      const q = new import_bullmq.Queue(Klass.queueName, __spreadValues({
        connection: this.connection
      }, this.opts.queueOpts));
      q.on("error", (err) => {
        HyperWorker.logger && HyperWorker.logger.error(`[error] ${err}`);
        HyperWorker.reportError(err);
      });
      const enqueue = (data, opts2) => q.add("job", data, opts2);
      Klass.enqueue = enqueue;
      Klass.queue = q;
      return q;
    });
    this.workers = workers;
  }
  async processJobs() {
    if (this.bmworkers) {
      throw new Error("already processing jobs");
    }
    if (!HyperWorker.logger) {
      throw new Error("logger is missing. did the app boot correctly?");
    }
    const { magenta, bold } = HyperWorker.logger.colors;
    const pref = bold(magenta("workers:"));
    if (!this.workers) {
      throw new Error("you need to wire into your worker first. call 'wireInto(<workers>)'");
    }
    this.bmworkers = this.workers.filter((Klass) => {
      if (Klass.producerOnly) {
        HyperWorker.logger.info(`${pref} producer only job: ${Klass.queueName}`);
        return false;
      }
      return true;
    }).map((Klass) => {
      debug2(`processing queue: ${Klass.queueName} from class ${Klass.name}`);
      return new import_bullmq.Worker(Klass.queueName, async (job) => {
        debug2(`processing job in: ${Klass.queueName}`);
        const w = new Klass();
        await w.perform(job);
      }, __spreadValues({
        connection: this.connection
      }, this.opts.workerOpts));
    });
    this.bmworkers.forEach((w) => {
      w.on("drained", (_job) => {
        HyperWorker.logger.info(`${pref} drained: ${w.name}`);
      });
      w.on("completed", (_job) => {
        HyperWorker.logger.info(`${pref} completed: ${w.name}`);
      });
      w.on("failed", (job, err) => {
        HyperWorker.logger.info(`${pref} failed: ${w.name} (${err})`);
        HyperWorker.reportError(err, { worker: w.name, jobId: job.id });
      });
      w.on("error", (err) => {
        HyperWorker.logger.error(`${pref} error: ${w.name} (${err})`);
        HyperWorker.reportError(err, { worker: w.name });
      });
    });
  }
  async stop() {
    if (this.bmworkers) {
      for (const w of this.bmworkers) {
        if (w) {
          w.connection.removeAllListeners("close");
          w.connection.removeAllListeners("error");
          w.removeAllListeners("error");
          w.removeAllListeners("completed");
          w.removeAllListeners("drained");
          await w.close();
        }
      }
    }
    if (this.bmqueues) {
      for (const q of this.bmqueues) {
        if (q) {
          q.connection.removeAllListeners("close");
          q.connection.removeAllListeners("error");
          q.removeAllListeners("error");
          q.removeAllListeners("completed");
          q.removeAllListeners("drained");
          await q.close();
        }
      }
    }
    await this.connection.quit();
    this.workers = void 0;
    await sleep(500);
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  BullmqBackend
});
